# -*- coding: utf-8 -*-

"""Test of the module :module:`tffpy.datasets`

.. moduleauthor:: Valentin Emiya
"""
import unittest
import numpy as np

from tffpy.datasets import get_mix, get_dataset
from tffpy.utils import snr


class TestGetMix(unittest.TestCase):

    def setUp(self):
        dataset = get_dataset()
        self.loc_source_list = list(dataset['localized'].keys())
        self.wb_source_list = list(dataset['wideband'].keys())

    def test_snr(self):
        loc_source = self.loc_source_list[0]
        wb_source = self.wb_source_list[0]
        for wb_to_loc_ratio_db in [-10, -3, 0, 6]:
            x_mix, dgt_params, signal_params, mask, x_loc, x_wb = \
                get_mix(loc_source=loc_source, wideband_src=wb_source,
                        wb_to_loc_ratio_db=wb_to_loc_ratio_db,
                        win_dur=128 / 8000, win_type='gauss',
                        hop_ratio=1 / 4, n_bins_ratio=4, n_iter_closing=2,
                        n_iter_opening=2, delta_mix_db=0, delta_loc_db=30,
                        closing_first=True, fig_dir=None)
            np.testing.assert_array_almost_equal(x_mix, x_loc + x_wb)
            np.testing.assert_almost_equal(snr(x_signal=x_wb, x_noise=x_loc),
                                           wb_to_loc_ratio_db)

